#include "LoadPPM.h"

/*

LoadPPM reads PPM files and transforms any type into 24 bit RGB

*/

LoadPPM::LoadPPM(unsigned char *&p_rgb_image, char *p_filename,
		 long &width, long &height)
{
    const int MAX_LINE = 6000;
    char text[MAX_LINE];
    bool binary_file;
    int ppm_type = -1, max_color_comp, i, gray;

    _error_number = 0;
    p_rgb_image = NULL;

    ifstream image_stream(p_filename, ios::binary);

    if (!image_stream.bad()) {

	// read PPM header and determine file format

	if (!GetNextPPMLine(image_stream, text, MAX_LINE)) {
	    _error_number = -1;	// empty file or comments only
	    goto PPM_abort;
	}

	if (text[0] == 'P') {

	    if (text[1] == '3')
		ppm_type = 1;	// ASCII PPM
	    if (text[1] == '6')
		ppm_type = 2;	// binary PPM
	}			// if

	if (ppm_type < 0) {
	    _error_number = -2;	// invalid PPM header
	    goto PPM_abort;
	}
	// read image resolution

	if (!GetNextPPMLine(image_stream, text, MAX_LINE)) {
	    _error_number = -4;	// no file resolution found
	    goto PPM_abort;
	}			// if

	width = atoi(text);

	if ((width < 1) || (width > USHRT_MAX)) {
	    _error_number = -5;	// invalid image width
	    goto PPM_abort;
	}			// if

	for (i = 0; i < strlen(text); i++)
	    if (text[i] == ' ')
		break;

	height = atoi(&(text[i + 1]));

	if ((height < 1) || (height > USHRT_MAX)) {
	    _error_number = -6;	// invalid image height
	    goto PPM_abort;
	}			// if

	if (!GetNextPPMLine(image_stream, text, MAX_LINE)) {
	    _error_number = -7;	// no max. color-component information found
	    goto PPM_abort;
	}			// if

	max_color_comp = atoi(text);

	if ((max_color_comp < 1) || (max_color_comp > USHRT_MAX)) {
	    _error_number = -8;	// invalid max_color_comp
	    goto PPM_abort;
	}			// if

	// allocate image memory

	p_rgb_image = new unsigned char[width * height * 3];
	if (p_rgb_image == NULL) {
	    _error_number = -9;	// unable to allocate memory
	    goto PPM_abort;
	}			// if

	if (ppm_type == 1) {

	    // read ASCII type PPM pixel information

	    if (max_color_comp == 255) {

		unsigned char *p_work_rgb_image = p_rgb_image;
		long counter = 0, size_image = width * height * 3;

		while ((!image_stream.eof()) && (counter < size_image)) {

		    GetNextPPMLine(image_stream, text, MAX_LINE);
		    char *p_work_text = text;

		    do {
			gray = GetNextNumber(p_work_text);
			if (gray >= 0) {
			    *p_work_rgb_image++ = gray;
			    counter++;
			}	// if

		    } while (gray >= 0);
		}		// while

	    }			// if
	    else {
		_error_number = -10;	// only 24 bit images are implemented yet for ASCII type PPM files
		goto PPM_abort;
	    }			// else
	}			// if
	else {

	    // read binary PPM pixel information

	    if (max_color_comp == 255) {
		image_stream.read((char*)p_rgb_image, width * height * 3);
	    }			// if
	    else {
		_error_number = -11;	// only 24 bit images are implemented yet for binary PPM files
		goto PPM_abort;
	    }			// else
	}			// else

    }				// if
    else {
	_error_number = -3;	// could not open file
	goto PPM_abort;
    }				// else

    return;

  PPM_abort:
    if (p_rgb_image)
	delete[]p_rgb_image;
    p_rgb_image = NULL;
}				// LoadPPM::LoadPPM

bool LoadPPM::GetNextPPMLine(ifstream & image_stream, char *p_line,
			     short size_buffer)
{
    do {
	image_stream.getline(p_line, size_buffer);
	if (image_stream.eof())
	    return false;
    } while (p_line[0] == '#');

    return true;
}				// LoadPPM::GetNextPPMLine
